/*
   +----------------------------------------------------------------------+
   | This library is free software; you can redistribute it and/or        |
   | modify it under the terms of the GNU Lesser General Public           |
   | License as published by the Free Software Foundation; either         |
   | version 2.1 of the License, or (at your option) any later version.   | 
   |                                                                      |
   | This library is distributed in the hope that it will be useful,      |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
   | Lesser General Public License for more details.                      | 
   |                                                                      |
   | You should have received a copy of the GNU Lesser General Public     |
   | License in the file LICENSE along with this library;                 |
   | if not, write to the                                                 | 
   |                                                                      |
   |   Free Software Foundation, Inc.,                                    |
   |   59 Temple Place, Suite 330,                                        |
   |   Boston, MA  02111-1307  USA                                        |
   +----------------------------------------------------------------------+
   | Authors: Hartmut Holzgraefe <hartmut@php.net>                        |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#include "php_mdbtools.h"

#if HAVE_MDBTOOLS


int le_MdbDatabase;

void MdbDatabase_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
  MdbHandle * resource = (MdbHandle *)(rsrc->ptr);

	  mdb_free_handle(resource);

}

int le_MdbTable;

void MdbTable_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
  my_MdbTableDef * resource = (my_MdbTableDef *)(rsrc->ptr);

	  int i;

	for (i = 0; i < resource->num_cols; i++) {
	  efree(resource->column_names[i]);
	  efree(resource->bound_values[i]);
	}
	efree(resource->column_names);
	efree(resource->bound_values);

	/* there is no free function for tables in the API?
	mdb_free_table(resource->table);
	*/

  efree(resource);
}
/* {{{ mdbtools_functions[] */
function_entry mdbtools_functions[] = {
	PHP_FE(mdb_open            , NULL)
	PHP_FE(mdb_close           , NULL)
	PHP_FE(mdb_version         , NULL)
	PHP_FE(mdb_tables          , NULL)
	PHP_FE(mdb_table_open      , NULL)
	PHP_FE(mdb_fetch_row       , NULL)
	PHP_FE(mdb_fetch_assoc     , NULL)
	PHP_FE(mdb_table_close     , NULL)
	PHP_FE(mdb_num_fields      , NULL)
	PHP_FE(mdb_num_rows        , NULL)
	PHP_FE(mdb_table_fields    , NULL)
	PHP_FE(mdb_type_name       , NULL)
	{ NULL, NULL, NULL }
};
/* }}} */


/* {{{ mdbtools_module_entry
 */
zend_module_entry mdbtools_module_entry = {
	STANDARD_MODULE_HEADER,
	"mdbtools",
	mdbtools_functions,
	PHP_MINIT(mdbtools),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN(mdbtools), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT(mdbtools),         /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN(mdbtools),   /* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO(mdbtools),
	"0.1", 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_MDBTOOLS
ZEND_GET_MODULE(mdbtools)
#endif


/* {{{ globals and ini entries */
/* }}} */


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(mdbtools)
{
	REGISTER_LONG_CONSTANT("MDB_VER_JET3", MDB_VER_JET3, 0);
	REGISTER_LONG_CONSTANT("MDB_VER_JET4", MDB_VER_JET4, 0);
	REGISTER_LONG_CONSTANT("MDB_BOOL", MDB_BOOL, 0);
	REGISTER_LONG_CONSTANT("MDB_BYTE", MDB_BYTE, 0);
	REGISTER_LONG_CONSTANT("MDB_INT", MDB_INT, 0);
	REGISTER_LONG_CONSTANT("MDB_LONGINT", MDB_LONGINT, 0);
	REGISTER_LONG_CONSTANT("MDB_MONEY", MDB_MONEY, 0);
	REGISTER_LONG_CONSTANT("MDB_FLOAT", MDB_FLOAT, 0);
	REGISTER_LONG_CONSTANT("MDB_DOUBLE", MDB_DOUBLE, 0);
	REGISTER_LONG_CONSTANT("MDB_SDATETIME", MDB_SDATETIME, 0);
	REGISTER_LONG_CONSTANT("MDB_TEXT", MDB_TEXT, 0);
	REGISTER_LONG_CONSTANT("MDB_OLE", MDB_OLE, 0);
	REGISTER_LONG_CONSTANT("MDB_MEMO", MDB_MEMO, 0);
	REGISTER_LONG_CONSTANT("MDB_REPID", MDB_REPID, 0);
	REGISTER_LONG_CONSTANT("MDB_NUMERIC", MDB_NUMERIC, 0);
	
	le_MdbDatabase = zend_register_list_destructors_ex(MdbDatabase_dtor, 
														 NULL, 
														 "MdbDatabase", 
														 module_number);
	
	le_MdbTable = zend_register_list_destructors_ex(MdbTable_dtor, 
														 NULL, 
														 "MdbTable", 
														 module_number);

	{
		mdb_init();

	}

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(mdbtools)
{
	mdb_exit();

  
	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION(mdbtools)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION(mdbtools)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(mdbtools)
{
	php_info_print_box_start(0);
	php_printf("<p>MDB data file access library</p>\n");
	php_printf("<p>Version 0.1alpha (2004-03-25)</p>\n");
	php_printf("<p><b>Authors:</b></p>\n");
  php_printf("<p>Hartmut Holzgraefe &lt;hartmut@php.net&gt; (lead)</p>\n");
	php_info_print_box_end();
	/* add your stuff here */

}
/* }}} */


/* {{{ proto resource MdbDatabase mdb_open(string path)
  Open an Access .mdb file for reading */
PHP_FUNCTION(mdb_open)
{
  MdbHandle * return_res;
  char * path = NULL;
  int path_len = 0;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "s", &path, &path_len) == FAILURE) return;
  {
	return_res = mdb_open(path);

	if (!return_res) RETURN_FALSE;

	ZEND_REGISTER_RESOURCE(return_value, return_res, le_MdbDatabase);
  }
}
/* }}} */


/* {{{ proto void mdb_close(resource MdbDatabase db)
  Close an open Access .mdb file */
PHP_FUNCTION(mdb_close)
{
  zval * db = NULL;
  int db_id = -1;
  MdbHandle * res_db;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &db) == FAILURE) return;

  if (db) {
	ZEND_FETCH_RESOURCE(res_db, MdbHandle *, &db, db_id, "MdbDatabase", le_MdbDatabase);
  }

  {
	 zval_dtor(db);
  }
}
/* }}} */


/* {{{ proto int mdb_version(resource MdbDatabase db)
  Get Jet Engine version number from .mdb file */
PHP_FUNCTION(mdb_version)
{
  zval * db = NULL;
  int db_id = -1;
  MdbHandle * res_db;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &db) == FAILURE) return;

  if (db) {
	ZEND_FETCH_RESOURCE(res_db, MdbHandle *, &db, db_id, "MdbDatabase", le_MdbDatabase);
  }

  {
	 RETURN_LONG(res_db->f->jet_version);
  }
}
/* }}} */


/* {{{ proto array mdb_tables(resource MdbDatabase db [, bool systabs])
  Get names of tables in an .mdb file */
PHP_FUNCTION(mdb_tables)
{
  zval * db = NULL;
  int db_id = -1;
  MdbHandle * res_db;
  zend_bool systabs = 0;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r|b", &db, &systabs) == FAILURE) return;

  if (db) {
	ZEND_FETCH_RESOURCE(res_db, MdbHandle *, &db, db_id, "MdbDatabase", le_MdbDatabase);
  }

  {
  int i;
  MdbCatalogEntry *entry;

  /* read the catalog */
  if (NULL == mdb_read_catalog (res_db, MDB_TABLE)) RETURN_FALSE;

  array_init(return_value);

  /* loop over each entry in the catalog */
  for (i=0; i < res_db->num_catalog; i++) {
	entry = g_ptr_array_index (res_db->catalog, i);
		
	if (!systabs && !strncmp("MSys", entry->object_name, 4)) {
		continue;
	}
	add_next_index_string(return_value, entry->object_name, 1);
  }
  }
}
/* }}} */


/* {{{ proto resource MdbTable mdb_table_open(resource MdbDatabase db, string table)
  Open a table in an .mdb file for reading */
PHP_FUNCTION(mdb_table_open)
{
  my_MdbTableDef * return_res = (my_MdbTableDef *)emalloc(sizeof(my_MdbTableDef));
  zval * db = NULL;
  int db_id = -1;
  MdbHandle * res_db;
  char * table = NULL;
  int table_len = 0;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "rs", &db, &table, &table_len) == FAILURE) return;

  if (db) {
	ZEND_FETCH_RESOURCE(res_db, MdbHandle *, &db, db_id, "MdbDatabase", le_MdbDatabase);
  }

  {
	int i, j;
	MdbCatalogEntry *entry;
	MdbColumn *col;
 
	mdb_read_catalog(res_db, MDB_TABLE);

	return_res->table = NULL;

	for (i=0;i<res_db->num_catalog;i++) {
	  entry = g_ptr_array_index(res_db->catalog,i);
	  if (!strcmp(entry->object_name, table)) {
		return_res->table = mdb_read_table(entry);
		if (!return_res->table) RETURN_FALSE;

		return_res->num_cols = return_res->table->num_cols;

		mdb_read_columns(return_res->table);

		mdb_rewind_table(return_res->table); 

		return_res->column_names = ecalloc(return_res->num_cols, sizeof(char *));
		return_res->bound_values = ecalloc(return_res->num_cols, sizeof(char *));

		for (j=0; j<return_res->num_cols; j++) {
		  col=g_ptr_array_index(return_res->table->columns, j);

		  return_res->column_names[j] = col->name;

		  return_res->bound_values[j] = emalloc(MDB_BIND_SIZE);
		  return_res->bound_values[j][0] = '\0';
		  mdb_bind_column(return_res->table, j+1, return_res->bound_values[j]);
		}
	  }
	}

	if (!return_res->table) RETURN_FALSE;

	ZEND_REGISTER_RESOURCE(return_value, return_res, le_MdbTable);
  }
}
/* }}} */


/* {{{ proto array mdb_fetch_row(resource MdbTable table)
  Get a table row as an enumerated array */
PHP_FUNCTION(mdb_fetch_row)
{
  zval * table = NULL;
  int table_id = -1;
  my_MdbTableDef * res_table;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &table) == FAILURE) return;

  if (table) {
	ZEND_FETCH_RESOURCE(res_table, my_MdbTableDef *, &table, table_id, "MdbTable", le_MdbTable);
  }

  {
	int i;

	if (!mdb_fetch_row(res_table->table)) {
		RETURN_FALSE;
	}

	array_init(return_value);

	for (i = 0; i < res_table->num_cols; i++) {
		add_next_index_string(return_value, res_table->bound_values[i], 1);
	}
  }
}
/* }}} */


/* {{{ proto array mdb_fetch_assoc(resource MdbTable table)
  Get a table row as an associative array */
PHP_FUNCTION(mdb_fetch_assoc)
{
  zval * table = NULL;
  int table_id = -1;
  my_MdbTableDef * res_table;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &table) == FAILURE) return;

  if (table) {
	ZEND_FETCH_RESOURCE(res_table, my_MdbTableDef *, &table, table_id, "MdbTable", le_MdbTable);
  }

  {
	int i;

	if (!mdb_fetch_row(res_table->table)) {
		RETURN_FALSE;
	}

	array_init(return_value);

	for (i = 0; i < res_table->num_cols; i++) {
		add_assoc_string(return_value, res_table->column_names[i], res_table->bound_values[i], 1);
	}
  }
}
/* }}} */


/* {{{ proto void mdb_table_close(resource MdbTable table)
  Close an opened table */
PHP_FUNCTION(mdb_table_close)
{
  zval * table = NULL;
  int table_id = -1;
  my_MdbTableDef * res_table;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &table) == FAILURE) return;

  if (table) {
	ZEND_FETCH_RESOURCE(res_table, my_MdbTableDef *, &table, table_id, "MdbTable", le_MdbTable);
  }

  {
	 zval_dtor(table);
  }
}
/* }}} */


/* {{{ proto int mdb_num_fields(resource MdbTable table)
  Get number of table columns */
PHP_FUNCTION(mdb_num_fields)
{
  zval * table = NULL;
  int table_id = -1;
  my_MdbTableDef * res_table;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &table) == FAILURE) return;

  if (table) {
	ZEND_FETCH_RESOURCE(res_table, my_MdbTableDef *, &table, table_id, "MdbTable", le_MdbTable);
  }

  {
	RETURN_LONG(res_table->table->num_cols);
  }
}
/* }}} */


/* {{{ proto int mdb_num_rows(resource MdbTable table)
  Get number of table rows */
PHP_FUNCTION(mdb_num_rows)
{
  zval * table = NULL;
  int table_id = -1;
  my_MdbTableDef * res_table;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &table) == FAILURE) return;

  if (table) {
	ZEND_FETCH_RESOURCE(res_table, my_MdbTableDef *, &table, table_id, "MdbTable", le_MdbTable);
  }

  {
	RETURN_LONG(res_table->table->num_rows);
  }
}
/* }}} */


/* {{{ proto array mdb_table_fields(resource MdbTable table)
  Get field type information for all fields in a table */
PHP_FUNCTION(mdb_table_fields)
{
  zval * table = NULL;
  int table_id = -1;
  my_MdbTableDef * res_table;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "r", &table) == FAILURE) return;

  if (table) {
	ZEND_FETCH_RESOURCE(res_table, my_MdbTableDef *, &table, table_id, "MdbTable", le_MdbTable);
  }

  {
	int i;
	zval *row;
	MdbColumn *col;

	array_init(return_value);
	
	for (i = 0; i < res_table->table->num_cols; i++)
	{
	   MAKE_STD_ZVAL(row);
	   array_init(row);

	   col = g_ptr_array_index (res_table->table->columns, i);

	   add_assoc_string(row, "name", col->name, 1);
	   add_assoc_long(row, "type", col->col_type);
	   add_assoc_long(row, "size", col->col_size);
	   add_assoc_long(row, "prec", col->col_prec);
	   add_assoc_long(row, "scale", col->col_scale);
	   add_assoc_long(row, "isfixed", col->is_fixed);

	   add_next_index_zval(return_value, row);
	}
  }
}
/* }}} */


/* {{{ proto string mdb_type_name(resource MdbDatabase db, int typecode)
  Look up the name for a field type */
PHP_FUNCTION(mdb_type_name)
{
  zval * db = NULL;
  int db_id = -1;
  MdbHandle * res_db;
  long typecode = 0;
  int argc = ZEND_NUM_ARGS();

  if (zend_parse_parameters(argc TSRMLS_CC, "rl", &db, &typecode) == FAILURE) return;

  if (db) {
	ZEND_FETCH_RESOURCE(res_db, MdbHandle *, &db, db_id, "MdbDatabase", le_MdbDatabase);
  }

  {
	RETURN_STRING(mdb_get_coltype_string (res_db->default_backend, typecode), 1);
  }
}
/* }}} */

#endif /* HAVE_MDBTOOLS */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
